package com.persagy.dmp.alarm.jms;

import cn.hutool.core.thread.ExecutorBuilder;
import com.persagy.dmp.rwd.model.DmpMessage;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * 消费消息线程池
 *
 * @author lixing
 * @version V1.0 2021/5/12 2:53 下午
 **/
@Component
@Slf4j
public class ConsumeMessageThreadPool {
    private final ExecutorService es;

    @Autowired
    private MessageProcesser messageProcesser;

    public ConsumeMessageThreadPool() {
        es = ExecutorBuilder.create()
                .setCorePoolSize(5)
                .setMaxPoolSize(10)
                .setWorkQueue(new LinkedBlockingQueue<>(10))
                .setHandler(new ThreadPoolExecutor.CallerRunsPolicy())
                .build();
    }

    public void consumeMessage(DmpMessage message) throws Exception {
        es.execute(() -> {
            messageProcesser.listen(message);
        });
    }
}
