package com.persagy.dmp.rwd.service;

import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.config.DmpParameterStorage;
import com.persagy.dmp.rwd.config.RwdConstants;
import com.persagy.dmp.rwd.entity.QSchema;
import com.persagy.dmp.rwd.entity.Schema;
import com.persagy.dmp.rwd.model.SchemaModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class SchemaService extends BaseService {

	@Autowired
	private CriteriaUtils criteriaUtils;

	private List<BooleanExpression> parse(ObjectNode criteria) {
		// TODO
		return null;
	}

	public PagedResponse<SchemaModel> query(JacksonCriteria criteria) {
		PagedResponse<SchemaModel> result = new PagedResponse<>();
		super.checkParam(result, RwdConstants.QUERY_GROUPCODE);
		if (!result.success()) {
			return result;
		}
		criteria.add(RwdConstants.OBJECT_GROUPCODE, DmpParameterStorage.getGroupCode());
		PagedResponse<Schema> resp = criteriaUtils.query(QSchema.schema, this::parse, criteria);
		result.setCount(resp.getCount());
		List<Schema> dataList = resp.getData();
		if (dataList != null && dataList.size() > 0) {
			List<SchemaModel> collect = dataList.stream().map(entity -> {
				SchemaModel model = entity.toModel();
				return model;
			}).collect(Collectors.toList());
			result.setData(collect);
		}
		return result;
	}

	@Transactional
	public MapResponse create(SchemaModel param) {
		MapResponse response = new MapResponse();
		// 1.校验必填参数
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		if (DmpParameterStorage.getGroupCode() == null) {
			response.setFail("集团编码不可为空");
			return response;
		}
		if (StrUtil.isBlank(param.getName())) {
			response.setFail("集团方案名称不可为空");
			return response;
		}
		// TODO
		// 1.根据名称和集团code查询方案,集团方案名称不能重复
		if (checkGroupSchemaNameRepeated(param.getName(), null)){
			response.setFail("集团方案名称不可重复");
			return response;
		}
		Schema entity = Schema.fromModel(param);
		if (entity.getId() == null) {
			entity.setId(UUID.randomUUID().toString().replace("-", ""));
		}
		entity.setGroupCode(DmpParameterStorage.getGroupCode());
		entity.setStatus(1);
		schemaRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}
	/***
	 * Description: 校验方案名称不可重复
	 * @param name : 集团方案名称
	 * @param schemaId : 集团方案id
	 * @return : java.lang.Boolean
	 * @author : lijie
	 * @date :2021/7/22 18:31
	 * Update By lijie 2021/7/22 18:31
	 */
	private Boolean checkGroupSchemaNameRepeated(String name,String schemaId){
		JacksonCriteria criteria = JacksonCriteria.newInstance();
		criteria.add(RwdConstants.OBJECT_GROUPCODE, DmpParameterStorage.getGroupCode());
		criteria.add(RwdConstants.OBJECT_NAME, name);
		PagedResponse<Schema> resp = criteriaUtils.query(QSchema.schema, this::parse, criteria);
		if (resp.getCount()<=0){
			return false;
		}
		if (StrUtil.isBlank(schemaId)){
			return true;
		}
		return !resp.getData().get(0).getId().equals(schemaId);
	}

	@Transactional
	public MapResponse update(SchemaModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		if (DmpParameterStorage.getGroupCode() == null) {
			response.setFail("集团编码不可为空");
			return response;
		}
		// TODO
		QSchema qt = QSchema.schema;
		String schemaId = param.getId();
		if (StrUtil.isBlank(schemaId)){
			response.setFail("集团方案id不能为空");
			return response;
		}
		Optional<Schema> opt = schemaRepository.findOne(qt.id.eq(schemaId).and(qt.groupCode.eq(DmpParameterStorage.getGroupCode())));
		if (!opt.isPresent()) {
			response.setFail("数据不存在");
			return response;
		}
		if (checkGroupSchemaNameRepeated(param.getName(), schemaId)){
			response.setFail("集团方案名称不可重复");
			return response;
		}
		Schema entity = opt.get();
		if (param.getName() != null) {
			entity.setName(param.getName());
		}
		if (param.getRemark() != null) {
			entity.setRemark(param.getRemark());
		}
		schemaRepository.save(entity);
		response.add("id", entity.getId());
		return response;
	}

	@Transactional
	public MapResponse delete(SchemaModel param) {
		MapResponse response = new MapResponse();
		super.checkParam(response, RwdConstants.QUERY_GROUPCODE);
		if (!response.success()) {
			return response;
		}
		// TODO
		schemaRepository.deleteById(param.getId());
		return response;
	}
}

